
package DE.fub.inf.JVM.JavaClass;


import DE.fub.inf.JVM.Constants;

import java.io.*;

/** 
 * This class is derived from the abstract 
 * <A HREF="DE.fub.inf.JVM.JavaClass.Constant.html">Constant</A> class 
 * and represents a reference to an interface method.
 *
 * @version 970922
 * @author  <A HREF="http://www.inf.fu-berlin.de/~dahm">M. Dahm</A>
 * @see     Constant
 */
public final class ConstantInterfaceMethodref extends Constant
{
  private short class_index,         // Reference to the class containing the
  name_and_type_index; // method and the method signature



  /**
   * Empty constructor, all attributes have to be defined via `setXXX'
   * methods. Use at your own risk.
   */
  public ConstantInterfaceMethodref() {
    tag = CONSTANT_InterfaceMethodref;
  }    
  /**
   * Initialize from another object.
   */
  public ConstantInterfaceMethodref(ConstantInterfaceMethodref c) {
    this(c.getClassIndex(), c.getNameAndTypeIndex());
  }    
  /**
   * Initialize instance from file data.
   *
   * @param DataInputStream
   * @throw IOException
   */
  ConstantInterfaceMethodref(DataInputStream file) throws IOException
  {
    this((short)file.readUnsignedShort(), (short)file.readUnsignedShort());
  }    
  /**
   * @param class_index Reference to the class containing the method
   * @param name_and_type_index and the method signature
   */
  public ConstantInterfaceMethodref(short class_index, 
				    short name_and_type_index)
  {
    this();
    this.class_index         = class_index;
    this.name_and_type_index = name_and_type_index;
  }    
  /**
   * Called by objects that are traversing the nodes of the tree implicitely
   * defined by the contents of a Java class. I.e., the hierarchy of methods,
   * fields, attributes, etc. spawns a tree of objects.
   *
   * @param v Visitor object
   */
  public void accept(Visitor v) {
    v.visitConstantInterfaceMethodref(this);
  }    
  /**
   * Dump constant interface method reference to file stream in binary format.
   *
   * @param file Output file stream
   * @throw IOException
   */ 
  public final void dump(DataOutputStream file) throws IOException
  {
    file.writeByte(tag);
    file.writeShort(class_index);
    file.writeShort(name_and_type_index);
  }    
  /**
   * @return Reference (index) to class of the method.
   */  
  public final short getClassIndex()       { return class_index; }    
  /**
   * @return Reference (index) to signature of the method.
   */  
  public final short getNameAndTypeIndex() { return name_and_type_index; }    
  /**
   * @param class_index.
   */
  public final void setClassIndex(short class_index) {
    this.class_index = class_index;
  }    
  /**
   * @param name_and_type_index.
   */
  public final void setNameAndTypeIndex(short name_and_type_index) {
    this.name_and_type_index = name_and_type_index;
  }    
  /**
   * @return String representation.
   */
  public final String toString()
  {
    return super.toString() + "(class_index = " + class_index +
      ", name_and_type_index = " + name_and_type_index + ")";
  }    
}
