
package DE.fub.inf.JVM.JavaClass;


import  DE.fub.inf.JVM.Constants;
import  java.io.*;

/**
 * This class represents a (PC offset, line number) pair, i.e. a line number in
 * the source that corresponds to a relative address in the byte code. This
 * is used for debugging purposes.
 *
 * @version 970922
 * @author  <A HREF="http://www.inf.fu-berlin.de/~dahm">M. Dahm</A>
 * @see     LineNumberTable
 */
public final class LineNumber
{
  private short start_pc;    // Program Counter (PC) corresponds to line
  private short line_number; // number in source file
 


  /**
   * Empty constructor, all attributes have to be defined via `setXXX'
   * methods. Use at your own risk.
   */
  public LineNumber() {}    
  /**
   * Initialize from another object.
   */
  public LineNumber(LineNumber c) {
    this(c.getStartPC(), c.getLineNumber());
  }    
  /**
   * Construct object from file stream.
   * @param file Input stream
   * @throw IOException
   */
  LineNumber(DataInputStream file) throws IOException
  {
    this((short)file.readUnsignedShort(), (short)file.readUnsignedShort());
  }    
  /**
   * @param start_pc Program Counter (PC) corresponds to
   * @param line_number line number in source file
   */
  public LineNumber(short start_pc, short line_number)
  {
    this.start_pc    = start_pc;
    this.line_number = line_number;
  }    
  /**
   * Called by objects that are traversing the nodes of the tree implicitely
   * defined by the contents of a Java class. I.e., the hierarchy of methods,
   * fields, attributes, etc. spawns a tree of objects.
   *
   * @param v Visitor object
   */
  public void accept(Visitor v) {
    v.visitLineNumber(this);
  }    
  /**
   * Dump line number/pc pair to file stream in binary format.
   *
   * @param file Output file stream
   * @throw IOException
   */ 
  public final void dump(DataOutputStream file) throws IOException
  {
    file.writeShort(start_pc);
    file.writeShort(line_number);
  }    
  /**
   * @return Corresponding source line
   */
  public final short getLineNumber() { return line_number; }    
  /**
   * @return PC in code
   */  
  public final short getStartPC() { return start_pc; }    
  /**
   * @param line_number.
   */
  public final void setLineNumber(short line_number) {
    this.line_number = line_number;
  }    
  /**
   * @param start_pc.
   */
  public final void setStartPC(short start_pc) {
    this.start_pc = start_pc;
  }    
  /**
   * @return String representation
   */ 
  public final String toString()
  {
    return "LineNumber(" + start_pc + ", " + line_number + ")";
  }    
}
